#include <TFile.h>
#include "TROOT.h"
#include "TSystem.h"
#include "TChain.h"
#include "TTree.h"

//_____________________
void FemtoDstQA_pau200_y2015(const char *flist = "test.list",
			     const char *ofile = "femtoQA_pau200_y2015.root") {
  std::cout << "Input file list = " << flist << '\n';
  if (!flist) {
    std::cout << "Input file list = NULL!\n";
    return;
  }
  
  //
  // Load libraries
  //
  gROOT->LoadMacro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
  loadSharedLibraries();
  gSystem->Load("libMinuit");
  gSystem->Load("StMuDSTMaker");
  gSystem->Load("StChain");
  gSystem->Load("StRefMultCorr");
  gSystem->Load("StFlowMaker");
  gSystem->Load("StFlowAnalysisMaker");
  gSystem->Load("StHbtMaker");
  gSystem->Load("StFemtoDstMaker");

  StChain           *chain = 0;
  StFemtoDstQAMaker *maker = 0;

  //
  // List of member links in the chain
  //
  chain = new StChain("StChain");
  maker = new StFemtoDstQAMaker("", flist, ".", 12000);
  maker->SetOutFileName(ofile);
  maker->SetRunIdParameters(1292, 16124017, 16159024); // pAu200 - production_pAu200_2015

  chain->Init();

  //
  // Loop over the links in the chain
  //
  int iret = 0;
  int nEvents = maker->GetNEvents();
  std::cout << "Entries in chain = " << nEvents << std::endl;
  
  for(int iev = 0; iev < nEvents; iev++) {
    chain->Clear();
    iret = chain->Make(iev);
    if(iret) {
      std::cout << "Error in Make occured. Error code: " << iret << std::endl;
      std::cout << "iev = " << iev << '\n';
      break;
    } 
  }
  chain->Finish();
  //Cleanup
  delete maker;
  delete chain;
}
